#include "stdafx.h"

static BOOL PrintVirtualizationStatus(HANDLE hToken);
static BOOL ChangeVirtualization(HANDLE hToken, BOOL bEnabled);


int _tmain(int argc, _TCHAR* argv[])
{
	HANDLE hToken = NULL;
	BOOL bResult = ::OpenProcessToken(
		::GetCurrentProcess(),
		TOKEN_QUERY | TOKEN_WRITE,
		&hToken
	);
	if( !bResult )
	{
		_tprintf(_T("OpenProcessToken() failed with code %d\n"), GetLastError());
		return 1;
	}

	if( !PrintVirtualizationStatus(hToken) )
	{
		::CloseHandle(hToken);
		return 1;
	}

	//
	//-- Enable virtualization
	//
	_tprintf(_T("\nEnabling virtualization...\n"));
	if( !ChangeVirtualization(hToken, TRUE) )
	{
		::CloseHandle(hToken);
		return 1;
	}

	if( !PrintVirtualizationStatus(hToken) )
	{
		::CloseHandle(hToken);
		return 1;
	}

	//
	//-- Disable virtualization
	//
	_tprintf(_T("\nDisabling virtualization...\n"));
	if( !ChangeVirtualization(hToken, FALSE) )
	{
		::CloseHandle(hToken);
		return 1;
	}

	if( !PrintVirtualizationStatus(hToken) )
	{
		::CloseHandle (hToken);
		return 1;
	}

	::CloseHandle(hToken);
	return 0;
}

BOOL PrintVirtualizationStatus(HANDLE hToken)
{
	_tprintf(_T("--------------\n"));
	DWORD dwVirtualizationAllowed = 0;
	DWORD dwRetLength = 0;
	BOOL bResult = ::GetTokenInformation(
		hToken,                           // __in       HANDLE TokenHandle
		TokenVirtualizationEnabled,       // __in       TOKEN_INFORMATION_CLASS TokenInformationClass,
		&dwVirtualizationAllowed,         // __out_opt  LPVOID TokenInformation,
		sizeof(dwVirtualizationAllowed),  // __in       DWORD TokenInformationLength,
		&dwRetLength                      // __out      PDWORD ReturnLength
	);
	if( !bResult )
	{
		_tprintf(_T("GetTokenInformation() failed with code %d\n"), GetLastError());
		return bResult;
	}
	_tprintf(_T("Virtualization is %s\n"), dwVirtualizationAllowed ? _T("allowed") : _T("not allowed"));

	DWORD dwVirtualizationEnabled = 0;
	dwRetLength = 0;
	bResult = ::GetTokenInformation(
		hToken,                           // __in       HANDLE TokenHandle
		TokenVirtualizationEnabled,       // __in       TOKEN_INFORMATION_CLASS TokenInformationClass,
		&dwVirtualizationEnabled,         // __out_opt  LPVOID TokenInformation,
		sizeof(dwVirtualizationEnabled),  // __in       DWORD TokenInformationLength,
		&dwRetLength                      // __out      PDWORD ReturnLength
	);
	if( !bResult )
	{
		_tprintf(_T("GetTokenInformation() failed with code %d\n"), GetLastError());
		return bResult;
	}

	_tprintf(_T("Virtualization is %s\n"), dwVirtualizationEnabled ? _T("enabled") : _T("disabled"));
	return bResult;
}

BOOL ChangeVirtualization(HANDLE hToken, BOOL bEnabled)
{
	DWORD dwVirtualizationEnabled = bEnabled ? 1 : 0;
	BOOL bResult = ::SetTokenInformation(
		hToken,                           // __in  HANDLE TokenHandle,
		TokenVirtualizationEnabled,       // __in  TOKEN_INFORMATION_CLASS TokenInformationClass,
		&dwVirtualizationEnabled,         // __in  LPVOID TokenInformation,
		sizeof(dwVirtualizationEnabled)   // __in  DWORD TokenInformationLength
	);

	if( !bResult )
	{
		_tprintf(_T("ChangeVirtualization(%d) failed with code %d\n"), bEnabled, GetLastError());
	}
	return bResult;
}